{ **************************************************************************** }
{                                                                              }
{ Smart Mobile Studio - Runtime Library                                        }
{                                                                              }
{ Copyright (c) Eric Grange, Creative IT. All rights reserved.                 }
{ Licensed to Optimale Systemer AS.                                            }
{                                                                              }
{ **************************************************************************** }
unit w3BufferedImage;

interface

uses w3system, w3graphics;

type
   TW3BufferedImage = class;

   TW3BufferedImageEvent = procedure (sender : TW3BufferedImage);

   TW3BufferedImage = class
      private
         FGraphicContext : TW3GraphicContext;
         FCanvas : TW3Canvas;
         FWidth : Integer;
         FHeight : Integer;
         FPrepared : Boolean;
         FOnRedraw : TW3BufferedImageEvent;

      public
         constructor Create(aWidth, aHeight : Integer);
         destructor Destroy; override;

         procedure Invalidate;
         procedure Prepare;

         // top-left anchor
         procedure Draw(dest : TW3Canvas; x, y : Float); overload;
         procedure Draw(dest : TW3Canvas; x, y : Float; const sourceRect : TRect); overload;

         // center anchor
         procedure DrawScaled(dest : TW3Canvas; x, y, scale : Float); overload;

         property Canvas : TW3Canvas read FCanvas;
         property Width : Integer read FWidth;
         property Height : Integer read FHeight;

         property Prepared : Boolean read FPrepared;
         property OnRedraw : TW3BufferedImageEvent read FOnRedraw write FOnRedraw;
   end;

implementation

constructor TW3BufferedImage.Create(aWidth, aHeight : Integer);
begin
   FWidth:=aWidth;
   FHeight:=aHeight;
   FGraphicContext:=TW3GraphicContext.Create(Null);
   FGraphicContext.Allocate(aWidth, aHeight);
   FCanvas:=TW3Canvas.Create(FGraphicContext);
end;

destructor TW3BufferedImage.Destroy;
begin
   FCanvas.Free;
   FGraphicContext.Free;
   inherited;
end;

procedure TW3BufferedImage.Invalidate;
begin
   FPrepared:=False;
end;

procedure TW3BufferedImage.Prepare;
begin
   if not Prepared then begin
      Canvas.Clear;
      if Assigned(OnRedraw) then
         FOnRedraw(Self);
      FPrepared:=True;
   end;
end;

procedure TW3BufferedImage.Draw(dest : TW3Canvas; x, y : Float);
begin
   Prepare;
   dest.DrawImageF(FGraphicContext.Handle, x, y);
end;

procedure TW3BufferedImage.Draw(dest : TW3Canvas; x, y : Float; const sourceRect : TRect);
begin
   Prepare;
   dest.DrawImageF(FGraphicContext.Handle,
                   sourceRect.Left, sourceRect.Top, sourceRect.Width, sourceRect.Height,
                   x, y, sourceRect.Width, sourceRect.Height);
end;

procedure TW3BufferedImage.DrawScaled(dest : TW3Canvas; x, y, scale : Float);
var
   w, h : Float;
begin
   Prepare;
   w:=Width*scale;
   h:=Height*scale;
   dest.DrawImageF(FGraphicContext.Handle, x-w*0.5, y-h*0.5, w, h);
end;


